package com.gesoft.czreal.common.base;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.support.annotation.Nullable;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;


import com.gesoft.czreal.BuildConfig;

import java.util.List;

public abstract class BaseActivity<A extends BaseActivity> extends AppCompatActivity {
    private String tag;
    private static final String SYSTEM_EXIT = BuildConfig.APPLICATION_ID.concat(".exit");
    // Activity 完全退出广播
    private ExitReceiver receiver;
    // 主线程handler，处理界面UI刷新
    private Handler mHandler;
    // 子线程handler，处理耗时任务
    private Handler tHandler;
    // AsyncTask 集合，处理GrpcTask生命周期
    private List<AsyncTask> list;
    // 当前Activity
    private A activity;
    private Toolbar toolbar;
    // 子线程
    private HandlerThread thread;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        tag = this.getClass().getSimpleName();
        activity = setActivity();
        //注册广播，用于退出程序
        IntentFilter filter = new IntentFilter();
        filter.addAction(SYSTEM_EXIT);
        receiver = new ExitReceiver();
        this.registerReceiver(receiver, filter);
        list = createList();

        setContentView();

        try {
            /*toolbar = (Toolbar) findViewById(R.id.toolbar);
            if (toolbar != null) {
                setSupportActionBar(toolbar);
                if (!(this instanceof HomeActivity)) {
                    getSupportActionBar().setDisplayHomeAsUpEnabled(true);
                    toolbar.setNavigationOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            onBackPressed();
                        }
                    });
                }
            }*/
        } catch (Exception e) {
        }
    }

    public HandlerThread getThread() {
        return thread;
    }

    public void setThread(HandlerThread thread) {
        this.thread = thread;
    }

    protected abstract A setActivity();

    protected abstract void setContentView();

    public Toolbar getToolbar() {
        return toolbar;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        this.unregisterReceiver(receiver);
        if (mHandler != null) {
            mHandler.removeCallbacksAndMessages(null);
        }

        if (tHandler != null) {
            tHandler.removeCallbacksAndMessages(null);
        }
        if (thread != null) {
            thread.quitSafely();
        }
        if (list != null && list.size() > 0) {
            for (AsyncTask asyncTask : list) {
                if (!asyncTask.isCancelled() && asyncTask.getStatus() != AsyncTask.Status.FINISHED) {
                    asyncTask.cancel(true);
                    list.remove(asyncTask);
                }
            }
            list.clear();
            list = null;
        }
    }

    @Override
    public void onBackPressed() {
        /*if (this instanceof HomeActivity) {
            // 完全退出
            Intent intent = new Intent(SYSTEM_EXIT);
            intent.putExtra("closeAll", 1);
            sendBroadcast(intent);//发送广播
        } else {
            super.onBackPressed();
        }*/
    }

    public A getActivity() {
        return activity;
    }

    public Context getContext() {
        return activity.getApplicationContext();
    }

    public String getTag() {
        return tag;
    }

    protected Handler getAutoLifeCyclemHandler() {
        return mHandler;
    }

    protected Handler getAutoLifeCycletHandler() {
        return tHandler;
    }

    protected void setAutoLifeCyclemHandler(Handler handler) {
        mHandler = handler;
    }

    protected void setAutoLifeCycletHandler(Handler handler) {
        tHandler = handler;
    }

    public List<AsyncTask> getAsyncTaskList() {
        return list;
    }

    protected abstract List<AsyncTask> createList();

    /**
     * 使用 AsyncTask 中的并行线程池
     *
     * @param run
     */
    protected void parallelExecute(Runnable run) {
        AsyncTask.THREAD_POOL_EXECUTOR.execute(run);
    }

    /**
     * 使用 AsyncTask 中的串行线程池
     *
     * @param run
     */
    protected void serialExecute(Runnable run) {
        AsyncTask.execute(run);
    }

    private class ExitReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            ((AppCompatActivity) context).finish();
        }
    }

    protected void addAutoLifecycleAsyncTask(AutoLifecycleAsyncTaskBuilder asyncTask) {
        list.add(asyncTask.build());
    }

    public interface AutoLifecycleAsyncTaskBuilder {
        AsyncTask build();
    }
}
