package com.gesoft.czreal.common.http;

import android.annotation.TargetApi;
import android.os.AsyncTask;
import android.os.Build;

import com.alibaba.fastjson.JSON;
import com.gesoft.czreal.BuildConfig;
import com.gesoft.czreal.common.base.BaseActivity;
import com.gesoft.czreal.common.base.BaseFragment;
import com.gesoft.czreal.common.bean.Method;
import com.gesoft.czreal.common.bean.MsgModel;
import com.gesoft.czreal.utils.LogUtil;


import java.lang.ref.WeakReference;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import okhttp3.Call;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;


public class HttpTask  extends AsyncTask<Void, Void, MsgModel> {


    private static final String TAG = HttpTask.class.getSimpleName();
    private static final String mHost;
    private static final int mPort = 8080;
    private static final String httpUrl;

    static {
        if (BuildConfig.DEBUG) {
            //mHost = "192.168.1.6";
            mHost = "192.168.0.64";
            //mHost = "192.168.0.64";
            //mHost = "192.168.0.116";
        } else {
            mHost = "aly.aja6.com";
        }
        httpUrl = String.format("http://%s:%d", mHost, mPort);
    }

    private WeakReference<BaseActivity> activityWeakReference;
    private WeakReference<BaseFragment> fragmentWeakReference;

    private Method                  inBean;
    private LoadingDialog           loadingDialog;
    private String                  msg;
    private TaskPostExecute         postExecute;
    private ActivityTaskPostExecute activityTaskPostExecute;
    private FragmentTaskPostExecute fragmentTaskPostExecute;
    private OkHttpClient okHttpClient = null;

    public HttpTask(Method inBean) {
        this.inBean = inBean;
        if (okHttpClient == null) {
            okHttpClient = getOkHttpClient();
        }
    }


    public HttpTask(BaseActivity activity, Method inBean, String msg, ActivityTaskPostExecute activityTaskPostExecute) {
        this(inBean);
        this.msg = msg;
        this.activityTaskPostExecute = activityTaskPostExecute;
        activityWeakReference = new WeakReference<>(activity);
    }

    public HttpTask(BaseFragment fragment, Method inBean, String msg, FragmentTaskPostExecute fragmentTaskPostExecute) {
        this(inBean);
        this.msg = msg;
        this.fragmentTaskPostExecute = fragmentTaskPostExecute;
        fragmentWeakReference = new WeakReference<>(fragment);
        activityWeakReference = new WeakReference<>((BaseActivity) fragment.getActivity());
    }




    @TargetApi(Build.VERSION_CODES.JELLY_BEAN_MR1)
    @Override
    protected void onPreExecute() {
        super.onPreExecute();
        if (msg == null) {
            return;
        }
        if (activityWeakReference != null &&
                activityWeakReference.get() != null &&
                !activityWeakReference.get().isFinishing() &&
                !activityWeakReference.get().isDestroyed()
                ) {
            LoadingDialog.Builder loadBuilder = new LoadingDialog.Builder(activityWeakReference.get()).setMessage(msg);
            loadingDialog = loadBuilder.create();
            loadingDialog.show();
        }
    }

    @Override
    protected MsgModel doInBackground(Void... nothing) {
        return service(inBean);
    }

    private MsgModel service(Method method) {
        MsgModel msgModel = new MsgModel();
        try{
            checkOkHttpClient();
            FormBody.Builder mFormBody = new FormBody.Builder();
            if (method.getMap() != null){
                Iterator<Map.Entry<String, String>> it = method.getMap().entrySet().iterator();
                while (it.hasNext()){
                    Map.Entry<String, String> en = it.next();
                    mFormBody.add(en.getKey(), en.getValue());
                }
            }
            RequestBody body = mFormBody.build();

            // 3 创建请求方式
            Request request = new Request.Builder().url(httpUrl + method.getMethodName()).post(body).build();

            // 4 执行请求操作
            Call call = okHttpClient.newCall(request);


            Response response = call.execute();
            if (response.isSuccessful()) {
                final String message = response.body().string();
                msgModel = JSON.parseObject(message, MsgModel.class);
            }
        }
        catch (Exception e){
            mLog(TAG, e.getMessage());
        }
        msgModel.setInMethod(method);
        return msgModel;
    }


    @TargetApi(Build.VERSION_CODES.JELLY_BEAN_MR1)
    @Override
    protected void onPostExecute(MsgModel msgModel) {
        if (loadingDialog != null) {
            loadingDialog.cancel();
        }
        mLog(TAG, "onPostExecute()");
        if (msgModel != null) {
            //生命周期管理
            //activity
            if (activityWeakReference != null) {
                BaseActivity baseActivity = activityWeakReference.get();
                if (baseActivity != null) {
                    if (!baseActivity.isFinishing() && !baseActivity.isDestroyed()) {
                        List asyncTaskList = baseActivity.getAsyncTaskList();
                        if (asyncTaskList != null) {
                            asyncTaskList.remove(HttpTask.this);
                        }
                        if (activityTaskPostExecute != null) {
                            mLog(TAG, "TaskPostExecute回调");
                            activityTaskPostExecute.onPostExecuteActivity(baseActivity, msgModel);
                            return;
                        }
                    }
                }
            }

            //fragment
            if (fragmentWeakReference != null) {
                BaseFragment fragment = fragmentWeakReference.get();
                if (fragment != null) {
                    if (!fragment.isRemoving() && !fragment.isDetached()) {
                        List asyncTaskList = fragment.getAsyncTaskList();
                        if (asyncTaskList != null) {
                            asyncTaskList.remove(HttpTask.this);
                        }
                        if (fragmentTaskPostExecute != null) {
                            mLog(TAG, "TaskPostExecute回调");
                            fragmentTaskPostExecute.onPostExecuteActivity(fragment, msgModel);
                            return;
                        }
                    }
                }
            }
        }
    }

    private OkHttpClient getOkHttpClient() {
        OkHttpClient mOkHttpClient = new OkHttpClient.Builder()
                .connectTimeout(10000, TimeUnit.MILLISECONDS)
                .readTimeout(10000, TimeUnit.MILLISECONDS)
                .build();
        return mOkHttpClient;
    }

    public void checkOkHttpClient(){
        if (okHttpClient == null) {
            mLog(TAG,"重新创建新连接！");
            getOkHttpClient();
        }
    }

    public void mLog(String message){
        LogUtil.mLog(TAG, message);
    }

    public void mLog(String TAG, String message){
        LogUtil.mLog(TAG, message);
    }
}
